﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using IMelody;
using IFretBoard;
using IGuitarConfig;
using Newtonsoft.Json;
using INoteLines;
using IMusicCommon;
using ITabs;
using NAudio.Midi;

namespace Guitar
{
    public partial class MainForm : Form
    {
         public MidiOut midi1;
        


        public MelodyEditMainForm EditMainForm;

        public Melody CurrentMelody;
        public String CurrentFile;
        public FretBoard FBoard;
        public GuitarConfig GConfig;
        public Tabs GTabs = new Tabs();
        NoteLines NoteLines;

        Bitmap FBBitmap;
        Bitmap LinesBitmap;


        int TactCount;
        int CurrentTact = 1;
        float CurrentTactPos;
        float TactStep;

        int TactsPerLine = 4; // !!!!!!!!!!!!!!!!!!!!!!!!
        //int TactSize = 4; // !!!!!!!!!!!!!!!!!!!!!!!!
        public int tabYoffs = 280;


        public List<int> HiglitedNotes = new List<int>();


        public MainForm()
        {
            InitializeComponent();
            CurrentMelody = new Melody();

            
            
            GConfig = new GuitarConfig();
            
            System.IO.StreamReader file2 = new System.IO.StreamReader("GuitarConf.json");
            string json = file2.ReadToEnd();
            GConfig = JsonConvert.DeserializeObject<GuitarConfig>(json);
            file2.Close();

            FBoard = new FretBoard(picFretboard.Size.Width, 7);
            //FBoard = new FretBoard(80, 3);
            NoteLines = new NoteLines(picNotes.Size.Width, picNotes.Size.Height - 50 - 100 /*minus tabs*/ );

            FBBitmap = new Bitmap(picFretboard.Size.Width, picFretboard.Size.Height);
            LinesBitmap = new Bitmap(picNotes.Size.Width, picNotes.Size.Height);


        }

        private void btnNewMelody_Click(object sender, EventArgs e)
        {
            

            if (EditMainForm == null)
            {
                EditMainForm = new MelodyEditMainForm(this);
            };
            EditMainForm.Show();

        }

        private void btnOpenMelody_Click(object sender, EventArgs e)
        {
            MelodyOpenDialog.ShowDialog();
            if (MelodyOpenDialog.FileName.Length > 0) {
                CurrentMelody = CurrentMelody.Load(MelodyOpenDialog.FileName);
                CurrentFile = MelodyOpenDialog.FileName;
                this.Text = CurrentFile;
                picNotes.Refresh();
            };
        }

        private void btnEditMelody_Click(object sender, EventArgs e)
        {
            if (EditMainForm == null)
            {
                EditMainForm = new MelodyEditMainForm(this);
                EditMainForm.MelodySize1.Value = (decimal)CurrentMelody.Ttactsize1;
                EditMainForm.MelodySize2.Value = (decimal)CurrentMelody.Ttactsize2;
            };
            EditMainForm.Text = CurrentFile;
            EditMainForm.Show();
        }

        private void picFretboard_Paint(object sender, PaintEventArgs e)
        {
            Graphics gr = Graphics.FromImage(FBBitmap);
            FBoard.DrawToBmp(gr);
            e.Graphics.DrawImage(FBBitmap, 0, 0);
        }

        private void toolStripButton1_Click(object sender, EventArgs e)
        {

            midi1 = new MidiOut(0);
            midi1.Volume = 65535;
            midi1.Send(MidiMessage.ChangePatch(26, 1).RawData);

            TactCount = CurrentMelody.GetTactCount();
            CurrentTact = 1 ;
            CurrentTactPos = 1;
            TactStep = 0.125f; // Must be calculated!

            timer1.Interval = (int)numSpeed.Value;
            timer1.Enabled = true;
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            
            int needreset = 1;
            for (int i = 0; i < CurrentMelody.notes.Count; i++)
            {
                if (CurrentMelody.notes[i].StartTime == CurrentTactPos &&
                    CurrentMelody.notes[i].Tact == CurrentTact &&
                    CurrentMelody.notes[i].MidiNote < 128)
                {
                    if (needreset == 1) {
                        FBoard.ResetHighlight();
                        HiglitedNotes.Clear();
                        needreset = 0;
                    };

                    HiglitedNotes.Add(i);

                    int gpos = GConfig.GetMinFret(CurrentMelody.notes[i].MidiNote);

                    int zstring = GConfig.GuitarNotes[gpos].GuitarString;
                    int zfret = GConfig.GuitarNotes[gpos].GuitarFret;
                    FBoard.HiglitedStrings[zstring - 1] = 1;
                    FBoard.HiglitedFret[zstring - 1] = zfret;
                    midi1.Send(MidiMessage.StartNote(CurrentMelody.notes[i].MidiNote, 127, 1).RawData);

         //           MidiFile f1 = new MidiFile("f1.mid");
         //           MidiEventCollection c1 = f1.Events;
                    
                };
            }; // end for


            CurrentTactPos = CurrentTactPos + TactStep;
            if (CurrentTactPos >= CurrentMelody.Ttactsize1+1)
            {
                CurrentTactPos = 1;
                CurrentTact++;
                if (CurrentTact > TactCount)
                {
                    CurrentTact = TactCount;
                    timer1.Enabled = false;
                    midi1.Close();
                };
            };

            picFretboard.Refresh();
            picNotes.Refresh();
        } // end of timer1

        private void picNotes_Paint(object sender, PaintEventArgs e)
        {
            Graphics gr = Graphics.FromImage(LinesBitmap);
            gr.Clear(Color.White);
            NoteLines.DrawToBMP(gr);

            
            int DisplayFirstTact = (((CurrentTact-1) / TactsPerLine) * TactsPerLine) + 1;


            float TactWidth = picNotes.Size.Width / TactsPerLine;  // for example - 100
            float NoteOffset = TactWidth / (CurrentMelody.Ttactsize1 + 1);           // 100 /5 =20,  pos - 20, 40, 60,80

            // tact lines
            Melody.MelodyNote tactline = new Melody.MelodyNote();
            tactline.MidiNote = 1000;

            for (int i = 1; i < TactsPerLine; i++) {
                NoteLines.DrawNote(gr, tactline, Color.Black, i * (int)TactWidth);
            };

            // tact number
            for (int i = 0; i < TactsPerLine; i++)
            {
                gr.DrawString(Convert.ToString(DisplayFirstTact + i ), SystemFonts.DialogFont, Brushes.Black, (int)(i * (int)TactWidth) + 10, 10);
            };


            // all notes
            for (int i = 0; i < CurrentMelody.notes.Count; i++) {
                if (CurrentMelody.notes[i].Tact >= DisplayFirstTact && CurrentMelody.notes[i].Tact <= DisplayFirstTact + TactsPerLine
                    && CurrentMelody.notes[i].MidiNote < 1000) {

                        float XPos = ((CurrentMelody.notes[i].Tact - DisplayFirstTact)) * TactWidth + (NoteOffset * CurrentMelody.notes[i].StartTime);
                    
                    NoteLines.DrawNote(gr, CurrentMelody.notes[i], Color.Black, (int)XPos);
                    // tabs numbers
                    int gpos = GConfig.GetMinFret(CurrentMelody.notes[i].MidiNote);

                    int zstring = GConfig.GuitarNotes[gpos].GuitarString;
                    int zfret = GConfig.GuitarNotes[gpos].GuitarFret;
                    string notename = MusicCommon.MidiNoteToName(CurrentMelody.notes[i].MidiNote);

                    GTabs.DrawTab(gr, NoteLines.Width, 120, NoteLines.offsX, tabYoffs, (int)XPos - 10, zstring, Convert.ToString(zfret) , Brushes.Black,14);
                    //GTabs.DrawTab(gr, NoteLines.Width, 120, NoteLines.offsX, tabYoffs /*+ 20*/, (int)XPos + 5, zstring, notename, Brushes.Blue,10);


                };
             
            };

            // higlited notes
            for (int i = 0; i < HiglitedNotes.Count; i++) {
                float XPos = ((CurrentMelody.notes[HiglitedNotes[i]].Tact - DisplayFirstTact)) * TactWidth + (NoteOffset * CurrentMelody.notes[HiglitedNotes[i]].StartTime);
                NoteLines.DrawNote(gr, CurrentMelody.notes[HiglitedNotes[i]], Color.Red, (int)XPos);
            };

            // tabs
            GTabs.DrawToBMP(gr, NoteLines.Width, 120, NoteLines.offsX, tabYoffs);

            
            e.Graphics.DrawImage(LinesBitmap, 0, 0);
        }

        private void MainForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Space || e.KeyCode == Keys.PageDown) {
                CurrentTact = CurrentTact + TactsPerLine;
                picNotes.Refresh();
            };
            if (e.KeyCode == Keys.PageUp)
            {
                CurrentTact = CurrentTact - TactsPerLine;
                if (CurrentTact < 1) {
                    CurrentTact = 1;
                };
                picNotes.Refresh();
            };

        }

        private void numTactPerLine_ValueChanged(object sender, EventArgs e)
        {
            TactsPerLine = (int)numTactPerLine.Value;
            picNotes.Refresh();
        }

        private void btnSaveBMP_Click(object sender, EventArgs e)
        {
            LinesBitmap.Save(Convert.ToString(CurrentTact) + ".bmp");
        } 

    }
}
